/*
******************************************************************************

   BUF debugging macros

   Copyright (C) 2001 Christoph Egger	[Christoph_Egger@t-online.de]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _BUF_INTERNAL_DEBUG_H
#define _BUF_INTERNAL_DEBUG_H


#include <ggi/internal/buf.h>

#include <stdio.h>
#include <stdarg.h>
#include <ggi/types.h>
#include <ggi/gg.h>



__BEGIN_DECLS

/* Exported variables */
#ifdef BUILDING_BUF
extern uint32     _bufDebugState;
extern int        _bufDebugSync;
#else
IMPORTVAR uint32  _bufDebugState;
IMPORTVAR int     _bufDebugSync;
#endif
__END_DECLS


/* Debugging types
 * bit 0 is reserved! */

#define BUFDEBUG_CORE		(1<<1)	/*   2 */
#define BUFDEBUG_MODE		(1<<2)	/*   4 */
#define BUFDEBUG_TARGET		(1<<3)	/*   8 */
#define BUFDEBUG_DRAW		(1<<4)	/*  16 */
#define BUFDEBUG_MISC		(1<<5)	/*  32 */
#define BUFDEBUG_LIBS		(1<<6)	/*  64 */
#define BUFDEBUG_EVENTS		(1<<7)	/* 128 */

#define BUFDEBUG_ALL	0xffffffff

#define DEBUG_INFO	__FILE__,__PRETTY_FUNCTION__,__LINE__

#ifdef DEBUG
#define LIB_ASSERT(x)	\
	if (!(x))	\
		fprintf(stderr,"LIBBUF:%s:%s:%d: assertion \"%s\" failed.\n", DEBUG_INFO, #x);
#else
#define LIB_ASSERT(x)	do{}while(0)
#endif


#ifdef __GNUC__

#ifdef DEBUG
#define BUFDPRINT(args...)	   if (_bufDebugState) { ggDPrintf(_bufDebugSync, "LibBUF",args); }
#define BUFDPRINT_CORE(args...)   if (_bufDebugState & BUFDEBUG_CORE) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#define BUFDPRINT_MODE(args...)   if (_bufDebugState & BUFDEBUG_MODE) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#define BUFDPRINT_TARGET(args...)  if (_bufDebugState & BUFDEBUG_TARGET) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#define BUFDPRINT_DRAW(args...)   if (_bufDebugState & BUFDEBUG_DRAW) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#define BUFDPRINT_MISC(args...)   if (_bufDebugState & BUFDEBUG_MISC) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#define BUFDPRINT_LIBS(args...)   if (_bufDebugState & BUFDEBUG_LIBS) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#define BUFDPRINT_EVENTS(args...) if (_bufDebugState & BUFDEBUG_EVENTS) { ggDPrintf(_bufDebugSync,"LibBUF",args); }
#else /* DEBUG */
#define BUFDPRINT(args...)		do{}while(0)
#define BUFDPRINT_CORE(args...)	do{}while(0)
#define BUFDPRINT_MODE(args...)	do{}while(0)
#define BUFDPRINT_TARGET(args...)	do{}while(0)
#define BUFDPRINT_DRAW(args...)	do{}while(0)
#define BUFDPRINT_MISC(args...)	do{}while(0)
#define BUFDPRINT_LIBS(args...)	do{}while(0)
#define BUFDPRINT_EVENTS(args...)	do{}while(0)
#endif /* DEBUG */

#else /* __GNUC__ */

__BEGIN_DECLS

static inline void BUFDPRINT(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_CORE(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_CORE) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_MODE(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_MODE) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_TARGET(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_TARGET) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_DRAW(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_DRAW) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_MISC(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_MISC) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_LIBS(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_LIBS) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

static inline void BUFDPRINT_EVENTS(const char *form,...)
{
#ifdef DEBUG
	if (_bufDebugState & BUFDEBUG_EVENTS) {
		va_list args;

		fprintf(stderr, "BUF: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bufDebugSync) fflush(stderr);
	}
#endif
}

__END_DECLS

#endif /* __GNUC__ */

#ifdef DEBUG
#define BUF_ASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"BUF:%s:%d: INTERNAL ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#define BUF_APPASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"BUF:%s:%d: APPLICATION ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#else /* DEBUG */
#define BUF_ASSERT(x,str)	do{}while(0)
#define BUF_APPASSERT(x,str)	do{}while(0)
#endif /* DEBUG */

#ifdef DEBUG
# define BUFD0(x)	x
#else
# define BUFD0(x)	/* empty */
#endif

#ifdef BUFDLEV
# if BUFDLEV == 1
#  define BUFD1(x)	x
#  define BUFD2(x)	/* empty */
#  define BUFD3(x)	/* empty */
# elif BUFDLEV == 2
#  define BUFD1(x)	x
#  define BUFD2(x)	x
#  define BUFD3(x)	/* empty */
# elif BUFDLEV > 2
#  define BUFD1(x)	x
#  define BUFD2(x)	x
#  define BUFD3(x)	x
# endif
#else
# define BUFD1(x)	/* empty */
# define BUFD2(x)	/* empty */
# define BUFD3(x)	/* empty */
#endif

#endif /* _BUF_INTERNAL_DEBUG_H */
