/*
******************************************************************************

   Demo of the OVL extension.

   Authors:	2001 Christoph Egger	[Christoph_Egger@t-online.de]
  
   This code is placed in the public domain and may be used freely for any
   purpose.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ggi/ggi.h>
#include <ggi/ovl.h>


static int do_demo(ggi_visual_t vis)
{
	int rc;
	ggiOvl_t ovl_sprite_ptr;
	ggiOvl_t ovl_sprite_cursor;

	ggi_color col;
	ggi_mode mode;
	int width, height;
	int inc, temp;
	char *pr;

	struct ggiGA_resource_props props_motor;
	struct ggiGA_resource_props props_carb;
	struct ggiGA_resource_props props_tank;

	ggiGA_resource_list reqlist = NULL;
	ggiGA_resource_list reslist = NULL;

	ggiOvlClearMotorProperties(&props_motor);
	ggiOvlClearCarbProperties(&props_carb);
	ggiOvlClearTankProperties(&props_tank);

	props_motor.sub.motor.grid_start.x =
	props_motor.sub.motor.grid_start.y = 0;

	props_tank.size.area.x = 174;
	props_tank.size.area.y = 57;
	props_tank.sub.tank.gt = GT_AUTO;

	printf("call ggiSetSimpleMode...");
	rc = ggiSetSimpleMode(vis, GGI_AUTO, GGI_AUTO, GGI_AUTO, GT_AUTO);
	printf("done -- rc: %i\n", rc);

	/* Attach the OVL extension to the visual using default reqlist */
	if (ggiOvlAttach(vis, &reqlist) < 0) {
		ggiPanic("Unable to attach OVL extension to visual\n");
	}	/* if */

	ggiGAGet(vis, &reslist);
	printf("Reslist after ggiSetMode\n");
	ggiGAanprintf(vis, reslist, 1024, NULL, &pr);
	printf("%s", pr);
	free(pr);
	ggiGAEmptyList(&reslist);

	printf("Add Sprite-Pointer...");
	ovl_sprite_ptr = ggiOvlCreate(vis, &props_motor, &props_carb,
				      &props_tank, GA_RT_SPRITE_POINTER);
	printf("done -- ovl_sprite_ptr: %p\n", ovl_sprite_ptr);


	props_tank.size.area.x = GGI_AUTO;
	props_tank.size.area.y = GGI_AUTO;
	props_tank.sub.tank.gt = GT_AUTO;

	printf("Add Sprite-Cursor...");
	ovl_sprite_cursor = ggiOvlCreate(vis, &props_motor, &props_carb,
					 &props_tank, GA_RT_SPRITE_CURSOR);
	printf("done -- ovl_sprite_cursor: %p\n", ovl_sprite_cursor);

	ggiGAGet(vis, &reslist);
	printf("Reslist after createsprite\n");
	ggiGAanprintf(vis, reslist, 1024, NULL, &pr);
        printf("%s", pr);
        free(pr);
	ggiGAEmptyList(&reslist);

	printf("Reqlist after ggiOvlCreateSprite\n");
	ggiGAanprintf(vis, reqlist, 1024, NULL, &pr);
        printf("%s", pr);
        free(pr);

	if (!ovl_sprite_ptr) {
		printf("create failed!\n");
		return -1;
	}	/* if */

	/* Set foreground color to white */
	col.r = 0xffff;
	col.g = 0xffff;
	col.b = 0xffff;
	ggiSetGCForeground(vis, ggiMapColor(vis, &col));

	/* Check what mode we actually got */
	ggiGetMode(vis, &mode);

	width = mode.visible.x;
	height = mode.visible.y;

	/* Draw a bunch of lines... */
	inc = height / 66;
	for (temp = 0; temp <= (height-inc); temp += inc) {
		ggiDrawLine(vis, 0, temp, width-1, height-temp);
	}	/* for */

	/* ...and flush the screen */
	ggiFlush(vis);

	ggiGAGet(vis, &reslist);
	printf("Reslist after ggiOvlCreateSprite\n");
	ggiGAanprintf(vis, reslist, 1024, NULL, &pr);
        printf("%s", pr);
        free(pr);
	ggiGAEmptyList(&reslist);

	{ /* GerBIlllLLLLLlllllzzzzzzzzzzzzssssssssss!!!! */
	  ggi_visual_t rennmaus, rennmaus2;
	  int rmf;
	  void *buff;

	  buff = malloc(9918);
	  rmf = open("./rennmaus.raw", O_RDONLY);
	  read(rmf, buff, 9918);
	  close(rmf);
	  rennmaus = ggiOpen("display-memory");
	  ggiSetGraphMode(rennmaus, 174, 57, 174, 57, GT_8BIT);
	  ggiSetColorfulPalette(rennmaus);
	  ggiPutBox(rennmaus, 0, 0, 174, 57, buff);
	  free(buff);
	  rennmaus2 = ggiOpen("display-memory");
	  ggiSetGraphMode(rennmaus2, 174, 57, 174, 57, GT_8BIT);
	  ggiSetColorfulPalette(rennmaus2);
	  ggiCrossBlit(rennmaus, 0, 0, 174, 57, vis, 0, 0);
	  ggiCrossBlit(rennmaus, 0, 0, 174, 57, rennmaus2, 0, 0);
	  ggiCrossBlit(rennmaus2, 0, 0, 174, 57, vis, 0, 57);
#if 0
	  ggiOvlSetImage(ovl_sprite_ptr, rennmaus2, 0, 0, NULL);
#else
	  ggiOvlSetImage(ovl_sprite_ptr, vis, 0, 0, NULL);
#endif
	  ggiClose(rennmaus);
	  ggiClose(rennmaus2);

	}

	ggiOvlShow(ovl_sprite_ptr);
	ggiOvlShow(ovl_sprite_cursor);


	for (;;) {

		static int hide = 0;
		ggi_event ev;

		struct timeval timeout={0,1000};

		ggiEventPoll(vis, emAll, &timeout);
		usleep(timeout.tv_usec);

		while (ggiEventsQueued(vis, emAll)) {
			ggiEventRead(vis, &ev, emAll);

			switch(ev.any.type) {
			case evKeyPress:
			case evKeyRepeat:

				switch(ev.key.sym) {

				case GIIK_Enter:
					hide ^= 1;

					if (hide) {
						ggiOvlHide(ovl_sprite_ptr);
					} else {
						ggiOvlShow(ovl_sprite_ptr);
					}	/* if */
					break;
				case GIIUC_Escape:
					goto cleanup;

				}	/* switch */
			}	/* switch */
		}	/* while */


		ggiFlush(vis);
	}	/* for */


	ggiOvlHide(ovl_sprite_ptr);
	ggiOvlHide(ovl_sprite_cursor);

cleanup:

	printf("Reqlist before ggiOvlDestroySprite (%p)\n", reqlist);
	ggiGAanprintf(vis, reqlist, 1024, NULL, &pr);
        printf("%s", pr);
        free(pr);

	rc = ggiOvlDestroyOvl(&ovl_sprite_ptr);
	printf("ret-val from ggiOvlDestroyOvl: %i\n", rc);

	rc = ggiOvlDestroyOvl(&ovl_sprite_cursor);
	printf("ret-val from ggiOvlDestroyOvl: %i\n", rc);

	printf("Reqlist after ggiOvlDestroyOvl (%p)\n", reqlist);
	ggiGAanprintf(vis, reqlist, 1024, NULL, &pr);
        printf("%s", pr);
        free(pr);

	rc = ggiGAReleaseList(vis, &reqlist);
	printf("ret-val from ggiGAReleaseList: %i\n", rc);

	/* Detach extension from visual */
	ggiOvlDetach(vis);

	return 0;
}	/* do_demo */



int main(void)
{
	ggi_visual_t vis;

	/* Initialize LibGGI */
	if (ggiInit() != 0) {
		fprintf(stderr, "Unable to initialize LibGGI\n");
		exit(1);
	}	/* if */
	
	/* Initialize OVL extension */
	if (ggiOvlInit() != 0) {
		ggiPanic("Unable to initialize LibOVL extension\n");
	}	/* if */

	/* Open the default visual */
	if ((vis = ggiOpen(NULL)) == NULL) {
		ggiPanic("Unable to open default visual\n");
	}	/* if */
	
	/* Turn on asynchronous mode (which should always be used) */
	ggiSetFlags(vis, GGIFLAG_ASYNC);
	

	/* Try the API functions */
	do_demo(vis);

	
	/* Close visual */
	ggiClose(vis);

	/* Deinitialize OVL extension */
	ggiOvlExit();

	/* Deinitialize LibGGI */
	ggiExit();

	return 0;
}	/* main */
